<?php

namespace Drupal\rest_role_restrict\EventSubscriber;

use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\HttpKernel\Event\ExceptionEvent;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class RestAccessDeniedSubscriber implements EventSubscriberInterface {

    public static function getSubscribedEvents() {
        return [
            KernelEvents::EXCEPTION => 'onAccessDenied',
        ];
    }

    public function onAccessDenied(ExceptionEvent $event) {
        $exception = $event->getThrowable();
        $request = $event->getRequest();

        // Only act on REST routes.
        if (
            $exception instanceof AccessDeniedHttpException &&
            str_starts_with($request->attributes->get('_route', ''), 'rest.')
        ) {
            $response = new JsonResponse([
                'message' => 'Access to the REST API is restricted to specific roles. Contact your site administrator.',
            ], 403);

            $event->setResponse($response);
        }
    }
}
